package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.CheckWorkDTO;
import com.diagbot.dto.MedicalRecordDTO;
import com.diagbot.entity.*;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.MedBehospitalTypeServiceImpl;
import com.diagbot.service.impl.MedicalRecordServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;


/**
 * @author songxl
 * @since 2021-05-11
 */
@Component
public class MedCheckWorkFacade {
    @Autowired
    SysUserFacade sysUserFacade;
    @Autowired
    private MedCheckInfoFacade medCheckInfoFacade;
    @Autowired
    private MedBehospitalTypeServiceImpl medBehospitalTypeServiceImpl;
    @Autowired
    private MedicalRecordServiceImpl medicalRecordServiceImpl;

    /**
     * @Author songxl
     * @Description 批量操作核查任务（增加和删除）
     * @Date 2021/5/11
     * @Param [medCheckWorkVO]
     * @Return java.lang.Boolean
     * @MethodName addCheck
     */
    public Boolean addCheckWork(MedCheckWorkAddVO medCheckWorkVO) {

        Date now = DateUtil.now();
        Long hospitalId = Long.parseLong(SysUserUtils.getCurrentHospitalID());
        Long principleId = Long.valueOf(SysUserUtils.getCurrentPrincipleID());
        QueryWrapper<SysUser> userQuer = new QueryWrapper<>();
        userQuer.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", principleId)
                .eq("status", 1);
        SysUser user = sysUserFacade.getOne(userQuer);
        String principleName = user.getLinkman();
        //操作类型（0-取消，1-添加）
        if (medCheckWorkVO.getCheckType() == 0) {
            QueryWrapper<MedCheckInfo> medicalRecordQe = new QueryWrapper<>();
            medicalRecordQe.eq("hospital_id", medCheckWorkVO.getHospitalId());
            medicalRecordQe.eq("is_deleted", IsDeleteEnum.N.getKey());
            if (medCheckWorkVO.getBehospitalCodeList().size() > 0) {
                medicalRecordQe.in("behospital_code", medCheckWorkVO.getBehospitalCodeList());
            }
//            List<MedCheckInfo> medCheckInfoList = medCheckInfoFacade.list(medicalRecordQe);
            List<MedCheckInfo> list = medCheckInfoFacade.list(medicalRecordQe);
            if (ListUtil.isEmpty(list)) {
                throw new CommonException(CommonErrorCode.UPDATE_INFO_FAIL, "存在未生成数据！");
            }

            medCheckInfoFacade.getBaseMapper().getUpdateBatch(medCheckWorkVO.getBehospitalCodeList());
        } else {
            //获取核查任务类型（0-科室任务 1-院级 2-质控科）
            switch (medCheckWorkVO.getWorkType()) {
                case 0:
                    //该病历没有核查的时候， 就添加核查记录
                    List<MedCheckInfo> medCheckInfoList = new ArrayList<>();
                    for (String behospitalCode : medCheckWorkVO.getBehospitalCodeList()) {
                        MedCheckInfo medCheckInfo = new MedCheckInfo();
                        medCheckInfo.setBehospitalCode(behospitalCode);
                        medCheckInfo.setGmtCreate(now);
                        medCheckInfo.setGmtModified(now);
                        medCheckInfo.setHospitalId(hospitalId);
                        medCheckInfo.setCheckType(medCheckWorkVO.getCheckType());
                        medCheckInfo.setJobCreator(principleId);
                        medCheckInfo.setJobCreatorName(principleName);
                        medCheckInfo.setJobCreateTime(now);
                        medCheckInfo.setJobType(medCheckWorkVO.getWorkType());
                        medCheckInfo.setJobDistributionTime(now);
                        medCheckInfoList.add(medCheckInfo);
                    }
                    medCheckInfoFacade.saveBatch(medCheckInfoList);
                    break;
                case 1:
                    break;
                default:
                    break;
            }
        }


//        List<MedCheckWork> checkWorkList = new ArrayList<>();

//        return medCheckWorkServiceImpl.saveBatch(checkWorkList, 10);
        return null;
    }

    /**
     * @Author songxl
     * @Description 获取每日住院患者的类型（出院、病危、抢救。。。）
     * @Date 2021/5/11
     * @Param [param]
     * @Return void
     * @MethodName execute
     */
    public void execute(String param) {
        JSONObject paramJson = JSONObject.parseObject(param);
        Long hospiatlId = paramJson.getLong("hospital");
        JSONObject typeValue = paramJson.getJSONObject("typeVal");
        JSONObject typeName = paramJson.getJSONObject("typeName");
        int searchSize = paramJson.getInteger("searchSize");
        int batchSize = paramJson.getInteger("batchSize");


        //定义批量插入的集合
        HashMap<String, MedBehospitalType> medBehospitalTypeMap = null;
        //1.判断患者类型表是否有记录
        int count = medBehospitalTypeServiceImpl.count();
        if (count > 0) {
            medBehospitalTypeMap = new HashMap<>();
            //2.1获取最近指定归档时间间隔的患者
            RecordTypeVO recordContentVO = new RecordTypeVO();
            recordContentVO.setHospitalId(hospiatlId);
            recordContentVO.setModeList(new ArrayList<String>(typeValue.keySet()));
            recordContentVO.setIsPlacefile(1l);//获取已归档的患者
            recordContentVO.setStartTime(DateUtil.formatDateTime(DateUtil.addMinutes(DateUtil.now(), -30)));
            recordContentVO.setEndTime(DateUtil.formatDateTime(DateUtil.now()));
            getPatientRecordType(medBehospitalTypeMap,
                    medicalRecordServiceImpl.getMedicalRecord(recordContentVO), typeValue, typeName, hospiatlId);
        } else {
            //2.2添加今天之前所有患者的类型
            //2.2.1获取所有数据数量
            medBehospitalTypeMap = new HashMap<>();
            RecordTypeVO recordContentVO = new RecordTypeVO();
            recordContentVO.setHospitalId(hospiatlId);
            recordContentVO.setModeList(new ArrayList<String>(typeValue.keySet()));
            recordContentVO.setIsPlacefile(1l);//获取已归档的患者
            int num = medicalRecordServiceImpl.getRecordCount(recordContentVO);
            if (num > 0) {

                if (num < searchSize) {
                    //执行一次
                    recordContentVO.setStart(0L);
                    recordContentVO.setEnd(num + 0L);
                    getPatientRecordType(medBehospitalTypeMap,
                            medicalRecordServiceImpl.getMedicalRecord(recordContentVO), typeValue, typeName, hospiatlId);
                }
                int i = num / searchSize + 1;
                for (int j = 0; j < i; j++) {
                    //5000个一次循环添加
                    recordContentVO.setStart(j * searchSize * 1L);
                    recordContentVO.setEnd(searchSize * 1L);
                    getPatientRecordType(medBehospitalTypeMap,
                            medicalRecordServiceImpl.getMedicalRecord(recordContentVO), typeValue, typeName, hospiatlId);
                }

            }

            //3.批量执行插入操作
            if (medBehospitalTypeMap.size() > 0) {
                List<MedBehospitalType> medBehospitalTypeList = new ArrayList<>(medBehospitalTypeMap.values());
                medBehospitalTypeServiceImpl.saveBatch(medBehospitalTypeList, batchSize);
            }

        }

    }

    /**
     * @Author songxl
     * @Description 获取患者类型存入批量插入map
     * @Date 2021/5/11
     * @Param [medBehospitalTypeMap, medicalRecordList, typeValue, typeName, hospiatlId]
     * @Return void
     * @MethodName getPatientRecordType
     */
    private void getPatientRecordType(HashMap<String, MedBehospitalType> medBehospitalTypeMap,
                                      List<MedicalRecordDTO> medicalRecordList,
                                      JSONObject typeValue,
                                      JSONObject typeName, Long hospiatlId) {

        if (medicalRecordList == null || medicalRecordList.isEmpty()) {
            return;
        }
        //1.遍历查询出来的结果
        for (MedicalRecordDTO medicalRecordDTO : medicalRecordList) {
            //2.判断批量插入map中有没有这个患者
            if (medBehospitalTypeMap.containsKey(medicalRecordDTO.getBehospitalCode())) {
                //3.比较这两次的分值大小
                int lval = medBehospitalTypeMap.get(medicalRecordDTO.getBehospitalCode()).getValue();
                int nval = typeValue.getInteger(medicalRecordDTO.getModeId() + "");
                if (nval > lval) {
                    medBehospitalTypeMap.get(medicalRecordDTO.getBehospitalCode()).setValue(nval);
                    medBehospitalTypeMap.get(medicalRecordDTO.getBehospitalCode())
                            .setBehospitalType(typeName.getString(medicalRecordDTO.getModeId() + ""));
                }
            } else {
                //4.获取分值记录这个患者
                MedBehospitalType medBehospitalType = new MedBehospitalType();
                medBehospitalType.setBehospitalCode(medicalRecordDTO.getBehospitalCode());
                medBehospitalType.setBehospitalType(typeName.getString(medicalRecordDTO.getModeId() + ""));
                medBehospitalType.setValue(typeValue.getInteger(medicalRecordDTO.getModeId() + ""));
                medBehospitalType.setCreateTime(DateUtil.now());
                medBehospitalType.setHospitalId(hospiatlId);
                medBehospitalTypeMap.put(medicalRecordDTO.getBehospitalCode(), medBehospitalType);
            }
        }
    }


    /**
     * 分页
     *
     * @param checkWorkPageVO
     * @return
     */
    public IPage<CheckWorkDTO> pageFac(CheckWorkPageVO checkWorkPageVO) {
        checkWorkPageSet(checkWorkPageVO);
        IPage<CheckWorkDTO> res = medCheckInfoFacade.getBaseMapper().getCheckWorkPage(checkWorkPageVO);
        return res;
    }

    private void checkWorkPageSet(CheckWorkPageVO checkWorkPageVO) {
        //入参验证
        //出院时间
        if (null != checkWorkPageVO && null != checkWorkPageVO.getStartDate()) {
            checkWorkPageVO.setStartDate(DateUtil.getFirstTimeOfDay(checkWorkPageVO.getStartDate()));
        }
        if (null != checkWorkPageVO && null != checkWorkPageVO.getEndDate()) {
            checkWorkPageVO.setEndDate(DateUtil.getFirstTimeOfDay(DateUtil.addDay(checkWorkPageVO.getEndDate(), 1)));
        }
        if (null != checkWorkPageVO && null != checkWorkPageVO.getStartDate() && null != checkWorkPageVO.getEndDate()) {
            if (DateUtil.after(checkWorkPageVO.getStartDate(), checkWorkPageVO.getEndDate())) {
                throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "出院时间的开始时间必须小于结束时间！");
            }
        }
        //住院天数
        if (null != checkWorkPageVO && null != checkWorkPageVO.getSDay() && null != checkWorkPageVO.getLDay()) {
            if (checkWorkPageVO.getSDay() > checkWorkPageVO.getLDay()) {
                throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "住院天数区间有误！");
            }
        }
        checkWorkPageVO.setHospitalId(Long.parseLong(SysUserUtils.getCurrentHospitalID()));
    }

}
